// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
import { __rest } from "tslib";
import { formatPreferHeader } from "./util";
import { createMetricsQueryResult, } from "../models/publicMetricsModels";
import { convertIntervalToTimeIntervalObject, convertTimespanToInterval, } from "../timespanConversion";
import { LogsQueryResultStatus, } from "../models/publicLogsModels";
/**
 * @internal
 */
export function convertRequestForQueryBatch(batch) {
    let id = 0;
    const requests = batch.map((query) => {
        const body = {
            workspaceId: query.workspaceId,
            query: query.query,
        };
        if (query["additionalWorkspaces"]) {
            body["workspaces"] = query["additionalWorkspaces"].map((x) => x);
        }
        if (query["timespan"]) {
            body["timespan"] = convertTimespanToInterval(query["timespan"]);
        }
        delete body["workspaceId"];
        delete body["includeQueryStatistics"];
        delete body["includeVisualization"];
        delete body["additionalWorkspaces"];
        delete body["serverTimeoutInSeconds"];
        const generatedRequest = {
            id: id.toString(),
            workspace: query.workspaceId,
            headers: formatPreferHeader(query),
            body,
        };
        ++id;
        return generatedRequest;
    });
    return {
        requests,
    };
}
/**
 * @internal
 */
export function convertResponseForQueryBatch(generatedResponse, rawResponse) {
    var _a;
    const fixApplied = fixInvalidBatchQueryResponse(generatedResponse, rawResponse);
    /* Sort the ids that are passed in with the queries, as numbers instead of strings
     * It is not guaranteed that service will return the responses for queries in the same order
     * as the queries are passed in
     */
    const responseList = generatedResponse.responses || [];
    const newResponse = (_a = responseList === null || responseList === void 0 ? void 0 : responseList.sort((a, b) => {
        let left = 0;
        if (a.id != null) {
            left = parseInt(a.id, 10);
        }
        let right = 0;
        if (b.id != null) {
            right = parseInt(b.id, 10);
        }
        return left - right;
    })) === null || _a === void 0 ? void 0 : _a.map((response) => convertBatchQueryResponseHelper(response));
    newResponse["__fixApplied"] = fixApplied;
    return newResponse;
}
/**
 * This is a workaround for a service bug that we're investigating. The 'body' column will occasionally come
 * back as a JSON string, instead of being a JSON object.
 *
 * (examples, with excess stuff trimmed)
 * Correct: `{"responses":[{"body":{"tables":[{"name":"PrimaryResult","columns":[{"name":"stringcolumn","type":"string"}],"rows":[["hello"]}`
 * Broken: `{"responses":[{"body":"{\"tables\":[{\"name\":\"PrimaryResult\",\"columns\":[{\"name\":\"stringcolumn\",\"type\":\"string\"}],\"rows\":[[\"hello\"]}`
 *
 * Issue here: https://github.com/Azure/azure-sdk-for-js/issues/15688
 *
 * @internal
 */
export function fixInvalidBatchQueryResponse(generatedResponse, rawResponse) {
    var _a;
    if (generatedResponse.responses == null) {
        return false;
    }
    let hadToFix = false;
    // the body here is incorrect, deserialize the correct one from the raw response itself.
    const parsedBody = JSON.parse(rawResponse.bodyAsText);
    // fix whichever responses are in this broken state (each query has it's own
    // response, so they're not all always broken)
    for (let i = 0; i < generatedResponse.responses.length; ++i) {
        if (((_a = generatedResponse.responses[i].body) === null || _a === void 0 ? void 0 : _a.error) != null) {
            continue;
        }
        // deserialize the raw response from the service, since we'll need index into it.
        generatedResponse.responses[i].body = parsedBody.responses[i].body;
        hadToFix = true;
    }
    return hadToFix;
}
/**
 * @internal
 */
export function convertRequestForMetrics(metricNames, queryMetricsOptions) {
    if (!queryMetricsOptions) {
        return {};
    }
    const { orderBy, aggregations, metricNamespace, timespan, granularity } = queryMetricsOptions, rest = __rest(queryMetricsOptions, ["orderBy", "aggregations", "metricNamespace", "timespan", "granularity"]);
    const obj = Object.assign({}, rest);
    if (timespan) {
        obj.timespan = convertTimespanToInterval(timespan);
    }
    if (orderBy) {
        obj.orderby = orderBy;
    }
    if (metricNames) {
        obj.metricnames = metricNames.join(",");
    }
    if (aggregations) {
        obj.aggregation = aggregations.join(",");
    }
    if (metricNamespace) {
        obj.metricnamespace = metricNamespace;
    }
    if (granularity) {
        obj.interval = granularity;
    }
    return obj;
}
/**
 * @internal
 */
export function convertResponseForMetrics(generatedResponse) {
    const metrics = generatedResponse.value.map((metric) => {
        const metricObject = Object.assign(Object.assign({}, metric), { name: metric.name.value, description: metric.displayDescription, timeseries: metric.timeseries.map((ts) => {
                var _a;
                return ({
                    data: ts.data,
                    metadataValues: (_a = ts.metadatavalues) === null || _a === void 0 ? void 0 : _a.map((mv) => {
                        var _a;
                        return (Object.assign(Object.assign({}, mv), { name: (_a = mv.name) === null || _a === void 0 ? void 0 : _a.value }));
                    }),
                });
            }) });
        delete metricObject.displayDescription;
        return metricObject;
    });
    // eslint-disable-next-line @typescript-eslint/no-unused-vars -- eslint doesn't recognize that the extracted variables are prefixed with '_' and are purposefully unused.
    const { resourceregion, value: _ignoredValue, interval, timespan } = generatedResponse, rest = __rest(generatedResponse, ["resourceregion", "value", "interval", "timespan"]);
    const obj = Object.assign(Object.assign({}, rest), { metrics, timespan: convertIntervalToTimeIntervalObject(timespan) });
    if (resourceregion) {
        obj.resourceRegion = resourceregion;
    }
    if (interval) {
        obj.granularity = interval;
    }
    return createMetricsQueryResult(obj);
}
/**
 * @internal
 */
export function convertRequestOptionsForMetricsDefinitions(options) {
    if (!options) {
        return {};
    }
    const { metricNamespace } = options, rest = __rest(options, ["metricNamespace"]);
    const obj = Object.assign({}, rest);
    if (metricNamespace) {
        obj.metricnamespace = metricNamespace;
    }
    return obj;
}
/**
 * @internal
 */
export function convertResponseForMetricsDefinitions(generatedResponse) {
    const definitions = generatedResponse === null || generatedResponse === void 0 ? void 0 : generatedResponse.map((genDef) => {
        const { name, dimensions, displayDescription, metricAvailabilities } = genDef, rest = __rest(genDef, ["name", "dimensions", "displayDescription", "metricAvailabilities"]);
        const response = Object.assign({}, rest);
        if (displayDescription) {
            response.description = displayDescription;
        }
        if (name === null || name === void 0 ? void 0 : name.value) {
            response.name = name.value;
        }
        const mappedMetricAvailabilities = metricAvailabilities === null || metricAvailabilities === void 0 ? void 0 : metricAvailabilities.map((genMetricAvail) => {
            return {
                granularity: genMetricAvail.timeGrain,
                retention: genMetricAvail.retention,
            };
        });
        if (mappedMetricAvailabilities) {
            response.metricAvailabilities = mappedMetricAvailabilities;
        }
        const mappedDimensions = dimensions === null || dimensions === void 0 ? void 0 : dimensions.map((dim) => dim.value);
        if (mappedDimensions) {
            response.dimensions = mappedDimensions;
        }
        return response;
    });
    return definitions;
}
/**
 * @internal
 */
export function convertResponseForMetricNamespaces(generatedResponse) {
    const namespaces = generatedResponse === null || generatedResponse === void 0 ? void 0 : generatedResponse.map((genDef) => {
        const { properties } = genDef, rest = __rest(genDef, ["properties"]);
        const response = Object.assign({}, rest);
        if (properties) {
            response.metricNamespaceName = properties.metricNamespaceName;
        }
        return response;
    });
    return namespaces;
}
/**
 * @internal
 */
export function convertGeneratedTable(table) {
    const dynamicsIndices = [];
    const datesIndices = [];
    // most columns convert on deserialization except for `dynamic` columns (basically JSON objects)
    // and 'datetime' (strings that are ISO8601 formatted dates)
    for (let i = 0; i < table.columns.length; ++i) {
        if (table.columns[i].type === "datetime") {
            datesIndices.push(i);
        }
        else if (table.columns[i].type === "dynamic") {
            dynamicsIndices.push(i);
        }
    }
    return Object.assign(Object.assign({}, table), { rows: table.rows.map((row) => {
            for (const dynamicIndex of dynamicsIndices) {
                try {
                    row[dynamicIndex] = JSON.parse(row[dynamicIndex]);
                }
                catch (_err) {
                    /* leave as is. */
                }
            }
            for (const dateIndex of datesIndices) {
                row[dateIndex] = new Date(row[dateIndex]);
            }
            return row;
        }), columnDescriptors: table.columns });
}
/**
 * @internal
 */
export function convertBatchQueryResponseHelper(response) {
    try {
        const parsedResponseBody = JSON.parse(response.body);
        return computeResultType(parsedResponseBody);
    }
    catch (e) {
        if (response.body)
            return computeResultType(response.body);
        else
            return {};
    }
}
export function computeResultType(generatedResponse) {
    var _a, _b;
    if (!generatedResponse.error) {
        const result = {
            visualization: generatedResponse.render,
            status: LogsQueryResultStatus.Success,
            statistics: generatedResponse.statistics,
            tables: ((_a = generatedResponse.tables) === null || _a === void 0 ? void 0 : _a.map((table) => convertGeneratedTable(table))) ||
                [],
        };
        return result;
    }
    else {
        if (generatedResponse.tables) {
            const result = {
                visualization: generatedResponse.render,
                status: LogsQueryResultStatus.PartialFailure,
                statistics: generatedResponse.statistics,
                partialTables: (_b = generatedResponse.tables) === null || _b === void 0 ? void 0 : _b.map((table) => convertGeneratedTable(table)),
                partialError: mapError(generatedResponse.error),
            };
            return result;
        }
        else {
            const errorInfo = mapError(generatedResponse.error);
            const result = Object.assign({ status: LogsQueryResultStatus.Failure }, errorInfo);
            return result;
        }
    }
}
export function mapError(error) {
    let innermostError = error;
    while (innermostError.innerError) {
        innermostError = innermostError.innerError;
    }
    return {
        name: "Error",
        code: error.code,
        message: `${error.message}.  ${innermostError.message}`,
    };
}
//# sourceMappingURL=modelConverters.js.map